# C API

The section below was generated automatically (devs: *gen_api.py*).
Please don't modify it by hand!

## Functions

### au_bc_dbg

```c
AU_PUBLIC void au_bc_dbg(const struct au_bc_storage *bcs,
                         const struct au_program_data *data);
```

Defined in *src/core/bc.h*.

Debugs an bytecode storage container

#### Arguments

 * **bcs:** the bytecode storage
 * **data:** program data

#### Return value

*none*

### au_bc_storage_del

```c
AU_PUBLIC void au_bc_storage_del(struct au_bc_storage *bc_storage);
```

Defined in *src/core/bc.h*.

Deinitializes an au_bc_storage instance

#### Arguments

 * **bc_storage:** instance to be deinitialized

#### Return value

*none*

### au_bc_storage_init

```c
AU_PUBLIC void au_bc_storage_init(struct au_bc_storage *bc_storage);
```

Defined in *src/core/bc.h*.

Initializes an au_bc_storage instance

#### Arguments

 * **bc_storage:** instance to be initialized

#### Return value

*none*

### au_binary_path

```c
AU_PRIVATE struct au_char_array au_binary_path();
```

Defined in *src/platform/path.h*.

Gets the path containing the currently running Aument executable

#### Arguments

*none*

#### Return value

*none*

### au_c_comp

```c
AU_PUBLIC void au_c_comp(struct au_c_comp_state *state,
                         const struct au_program *program,
                         const struct au_c_comp_options *options,
                         struct au_cc_options *cc);
```

Defined in *src/compiler/c_comp.h*.

Compiles a program into file specified by an au_c_comp_state instance

#### Arguments

 * **state:** An empty zero-initialized au_c_comp_state object
 * **program:** An au_program object generated by the parser
 * **options:** Additional C code generation options
 * **cc:** An optional pointer to an au_cc_options object. This will be used to pass additional arguments required for compilation.

#### Return value

*none*

### au_c_comp_state_del

```c
AU_PUBLIC void au_c_comp_state_del(struct au_c_comp_state *state);
```

Defined in *src/compiler/c_comp.h*.

Deinitializes an au_c_comp_state instance

#### Arguments

 * **state:** instance to be initialized

#### Return value

*none*

### au_cc_options_default

```c
AU_PUBLIC void au_cc_options_default(struct au_cc_options *cc);
```

Defined in *src/platform/cc.h*.

Initializes an au_cc_options instance with default parameters

#### Arguments

 * **cc:** instance to be initialized

#### Return value

*none*

### au_cc_options_del

```c
AU_PUBLIC void au_cc_options_del(struct au_cc_options *cc);
```

Defined in *src/platform/cc.h*.

Deinitializes an au_cc_options instance

#### Arguments

 * **cc:** instance to be destroyed

#### Return value

*none*

### au_class_interface_deref

```c
AU_PRIVATE void
au_class_interface_deref(struct au_class_interface *interface);
```

Defined in *src/core/rt/au_class.h*.

Decreases reference count of an au_class_interface instance. This struct is owned by a au_class_interface_ptr_array and should only be called when destroying it inside a au_class_interface_ptr_array.

#### Arguments

*none*

#### Return value

*none*

### au_class_interface_ref

```c
AU_PRIVATE void
au_class_interface_ref(struct au_class_interface *interface);
```

Defined in *src/core/rt/au_class.h*.

Increases reference count of an au_class_interface instance. This struct is owned by a au_class_interface_ptr_array and should only be called when initializing it inside a au_class_interface_ptr_array.

#### Arguments

*none*

#### Return value

*none*

### au_dispatch_func_del

```c
AU_PUBLIC void au_dispatch_func_del(struct au_dispatch_func *fn);
```

Defined in *src/core/fn/main.h*.

Deinitializes an au_dispatch_func instance

#### Arguments

 * **fn:** instance to be initialized

#### Return value

*none*

### au_extern_module_add_fn

```c
static AU_UNUSED inline int
au_extern_module_add_fn(au_extern_module_t p_data, const char *name,
                        au_extern_func_t func, int32_t num_args);
```

Defined in *src/lib/module.h*.

Declares an exported function in the external module

#### Arguments

 * **p_data:** the module
 * **name:** null-terminated string representing the name of the function
 * **func:** the pointer to the external function
 * **num_args:** the number of arguments the external function takes

#### Return value

0 if a function by that name already exists, 1 if successful

### au_extern_module_new

```c
static AU_UNUSED inline au_extern_module_t au_extern_module_new();
```

Defined in *src/lib/module.h*.

Creates a new external module

#### Arguments

*none*

#### Return value

a reference to the module

### au_fatal

```c
AU_PUBLIC AU_NO_RETURN void au_fatal(const char *fmt, ...);
```

Defined in *src/core/rt/exception.h*.

Print fatal exception and abort

#### Arguments

 * **fmt:** printf formatted string

#### Return value

*none*

### au_fatal_index

```c
AU_PUBLIC AU_NO_RETURN void au_fatal_index(const void *array, size_t idx,
                                           size_t len);
```

Defined in *src/core/rt/exception.h*.

Print index error and abort

#### Arguments

 * **array:** the array
 * **idx:** the index the code is accessing
 * **len:** the length of the array

#### Return value

*none*

### au_fn_call_internal

```c
static AU_UNUSED au_value_t
au_fn_call_internal(const struct au_fn *fn, struct au_vm_thread_local *tl,
                    const struct au_program_data *p_data,
                    const au_value_t *args, int *is_native);
```

Defined in *src/core/fn/call.h*.

Calls another aument function. If the called function lies in another module, it will recursively search for the real module that holds the function.

#### Arguments

 * **fn:** function to be called
 * **tl:** current thread-local object
 * **p_data:** the program data containing the function
 * **args:** the arguments passed to the function

#### Return value

the return value of the function

### au_fn_del

```c
AU_PUBLIC void au_fn_del(struct au_fn *fn);
```

Defined in *src/core/fn/main.h*.

Deinitializes an au_fn instance

#### Arguments

 * **fn:** instance to be initialized

#### Return value

*none*

### au_fn_fill_class_cache

```c
AU_PRIVATE void
au_fn_fill_class_cache(struct au_fn *fn,
                       const struct au_program_data *current_p_data);
```

Defined in *src/core/fn/main.h*.

Resolves references to classes imported from external modules. This function is called on every import and must NOT be used in functions shared across threads.

#### Arguments

 * **fn:** an au_fn function
 * **current_p_data:** the au_program_data instance that the function belongs to.

#### Return value

*none*

### au_fn_fill_import_cache

```c
AU_PRIVATE void
au_fn_fill_import_cache(struct au_fn *fn, size_t fn_idx_cached,
                        const struct au_program_data *p_data_cached);
```

Defined in *src/core/fn/main.h*.

Fills a cached reference to a function in an external module, and a reference to module itself, into an au_fn instance. This function is called on every import and must NOT be used in functions shared across threads.

#### Arguments

 * **fn:** the imported function. This function must have the type AU_FN_IMPORTER

#### Return value

*none*

### au_fn_num_args

```c
static inline int32_t au_fn_num_args(const struct au_fn *fn);
```

Defined in *src/core/fn/main.h*.

Returns the number of arguments the function takes.

#### Arguments

 * **fn:** the function

#### Return value

the number of arguments

### au_get_cc

```c
AU_PRIVATE char *au_get_cc();
```

Defined in *src/platform/cc.h*.

Returns the path of the C compiler

#### Arguments

*none*

#### Return value

*none*

### au_hash

```c
AU_PRIVATE au_hash_t au_hash(const uint8_t *str, const size_t len);
```

Defined in *src/core/hash.h*.

Hashes a chunk of memory `str` with length `len`

#### Arguments

*none*

#### Return value

*none*

### au_imported_func_del

```c
AU_PUBLIC void au_imported_func_del(struct au_imported_func *fn);
```

Defined in *src/core/fn/main.h*.

Deinitializes an au_imported_func instance

#### Arguments

 * **fn:** instance to be initialized

#### Return value

*none*

### au_imported_module_del

```c
AU_PUBLIC void au_imported_module_del(struct au_imported_module *data);
```

Defined in *src/core/program.h*.

Deinitializes an au_imported_module instance

#### Arguments

 * **data:** instance to be deinitialized

#### Return value

*none*

### au_imported_module_init

```c
AU_PUBLIC void au_imported_module_init(struct au_imported_module *data);
```

Defined in *src/core/program.h*.

Initializes an au_imported_module instance

#### Arguments

 * **data:** instance to be initialized

#### Return value

*none*

### au_lexer_del

```c
AU_PUBLIC void au_lexer_del(struct au_lexer *l);
```

Defined in *src/core/parser/lexer.h*.

Deinitializes a lexer instance

#### Arguments

 * **l:** instance to be deinitialized

#### Return value

*none*

### au_lexer_init

```c
AU_PUBLIC void au_lexer_init(struct au_lexer *l, const char *src,
                             size_t len);
```

Defined in *src/core/parser/lexer.h*.

Initializes a lexer instance

#### Arguments

 * **l:** instance to be initialized
 * **src:** pointer to source code to be lexed
 * **len:** byte-size length of the source code

#### Return value

*none*

### au_lexer_next

```c
AU_PUBLIC struct au_token au_lexer_next(struct au_lexer *l);
```

Defined in *src/core/parser/lexer.h*.

Gets the next token in the lexer stream

#### Arguments

*none*

#### Return value

*none*

### au_lexer_peek

```c
AU_PUBLIC struct au_token au_lexer_peek(struct au_lexer *l, int lh_pos);
```

Defined in *src/core/parser/lexer.h*.

Look ahead the next token in the lexer stream without consuming it

#### Arguments

*none*

#### Return value

*none*

### au_mmap_del

```c
AU_PUBLIC void au_mmap_del(struct au_mmap_info *info);
```

Defined in *src/platform/mmap.h*.

Deinitializes an au_mmap_info instance

#### Arguments

 * **info:** instance to be deinitialized

#### Return value

*none*

### au_mmap_read

```c
AU_PUBLIC int au_mmap_read(const char *path, struct au_mmap_info *info);
```

Defined in *src/platform/mmap.h*.

Loads a file into memory and stores a reference into a au_mmap_info instance

#### Arguments

 * **path:** path of file
 * **info:** info

#### Return value

1 if success, 0 if errored

### au_parse

```c
AU_PUBLIC struct au_parser_result au_parse(const char *src, size_t len,
                                           struct au_program *program);
```

Defined in *src/core/parser/parser.h*.

Parses source code into an au_program instance

#### Arguments

 * **src:** source code string
 * **len:** the bytesize len of the source code
 * **program:** output into a program

#### Return value

1 if parsed successfully, 0 if an error occurred

### au_perror

```c
AU_PUBLIC AU_NO_RETURN void au_perror(const char *msg);
```

Defined in *src/core/rt/exception.h*.

Print fatal exception from perror and exit program

#### Arguments

 * **msg:** what the program was trying to do when the error occured

#### Return value

*none*

### au_print_interpreter_error

```c
AU_PUBLIC void
au_print_interpreter_error(struct au_interpreter_result type,
                           struct au_error_location loc);
```

Defined in *src/core/int_error/error_printer.h*.

Prints an interpreter error

#### Arguments

*none*

#### Return value

*none*

### au_print_parser_error

```c
AU_PUBLIC void au_print_parser_error(struct au_parser_result res,
                                     struct au_error_location loc);
```

Defined in *src/core/int_error/error_printer.h*.

Prints a parser error

#### Arguments

*none*

#### Return value

*none*

### au_program_data_add_data

```c
AU_PRIVATE int au_program_data_add_data(struct au_program_data *p_data,
                                        au_value_t value, uint8_t *v_data,
                                        size_t v_len);
```

Defined in *src/core/program.h*.

Adds constant value data into a au_program_data instance

#### Arguments

 * **p_data:** au_program_data instance
 * **value:** au_value_t representation of constant
 * **v_data:** byte array of internal constant data
 * **v_len:** size of `v_data`

#### Return value

index of the data

### au_program_data_del

```c
AU_PUBLIC void au_program_data_del(struct au_program_data *data);
```

Defined in *src/core/program.h*.

Deinitializes an au_program_data instance

#### Arguments

 * **data:** instance to be deinitialized

#### Return value

*none*

### au_program_data_init

```c
AU_PUBLIC void au_program_data_init(struct au_program_data *data);
```

Defined in *src/core/program.h*.

Initializes an au_program_data instance

#### Arguments

 * **data:** instance to be initialized

#### Return value

*none*

### au_program_dbg

```c
AU_PUBLIC void au_program_dbg(const struct au_program *p);
```

Defined in *src/core/program.h*.

Debugs an au_program instance

#### Arguments

*none*

#### Return value

*none*

### au_program_del

```c
AU_PUBLIC void au_program_del(struct au_program *p);
```

Defined in *src/core/program.h*.

Initializes an au_program instance

#### Arguments

 * **p:** instance to be deinitialized

#### Return value

*none*

### au_program_import_del

```c
AU_PUBLIC void au_program_import_del(struct au_program_import *data);
```

Defined in *src/core/program.h*.

Deinitializes an au_program_import instance

#### Arguments

 * **data:** instance to be deinitialized

#### Return value

*none*

### au_spawn

```c
AU_PUBLIC int au_spawn(struct au_str_array *args);
```

Defined in *src/platform/spawn.h*.

Spawns a program with arguments specified

#### Arguments

*none*

#### Return value

*none*

### au_spawn_cc

```c
AU_PUBLIC int au_spawn_cc(struct au_cc_options *cc, char *output_file,
                          char *input_file);
```

Defined in *src/platform/cc.h*.

Spawns a C compiler with specified options, input file and output file

#### Arguments

 * **cc:** options passed into the compiler
 * **output_file:** path to the output file
 * **input_file:** path to the input file

#### Return value

exit code of the C compiler

### au_split_path

```c
AU_PRIVATE int au_split_path(const char *path, char **file, char **wd);
```

Defined in *src/platform/path.h*.

Splits a path into file and directory components

#### Arguments

 * **path:** the path to be split
 * **file:** output pointer to the file component
 * **wd:** output pointer to the directory component

#### Return value

1 if successful, 0 if failed

### au_string_add

```c
AU_PUBLIC struct au_string *au_string_add(const struct au_string *left,
                                          const struct au_string *right);
```

Defined in *src/core/rt/au_string.h*.

Creates an au_string from concatenating 2 au_string(s)

#### Arguments

 * **left:** First string
 * **right:** Second string

#### Return value

Concatenation of `left` and `right`

### au_string_cmp

```c
static AU_UNUSED inline int au_string_cmp(const struct au_string *left,
                                          const struct au_string *right);
```

Defined in *src/core/rt/au_string.h*.

Compares 2 au_string instances

#### Arguments

 * **left:** First string
 * **right:** Second string

#### Return value

Comparison of `left` and `right`

### au_string_cmp_cstr

```c
static AU_UNUSED inline int au_string_cmp_cstr(const struct au_string *str,
                                               const char *cstr);
```

Defined in *src/core/rt/au_string.h*.

Compares an au_string instance with a zero-terminated string

#### Arguments

 * **str:** au_string instance
 * **cstr:** C string

#### Return value

Comparison of `left` and `right`

### au_string_from_const

```c
AU_PUBLIC struct au_string *au_string_from_const(const char *s,
                                                 size_t len);
```

Defined in *src/core/rt/au_string.h*.

Creates an au_string from a constant array of chars

#### Arguments

 * **s:** Pointer to the array of chars
 * **len:** Byte size of the string

#### Return value

*none*

### au_tmpfile_close

```c
AU_PUBLIC void au_tmpfile_close(struct au_tmpfile *tmp);
```

Defined in *src/platform/tmpfile.h*.

Closes an au_tmpfile instance

#### Arguments

 * **tmp:** instance to be closed

#### Return value

*none*

### au_tmpfile_del

```c
AU_PUBLIC void au_tmpfile_del(struct au_tmpfile *tmp);
```

Defined in *src/platform/tmpfile.h*.

Deinitializes an au_tmpfile instance

#### Arguments

 * **tmp:** instance to be deinitialized

#### Return value

*none*

### au_tmpfile_exec

```c
AU_PUBLIC int au_tmpfile_exec(struct au_tmpfile *tmp);
```

Defined in *src/platform/tmpfile.h*.

Creates an empty executable file

#### Arguments

*none*

#### Return value

*none*

### au_tmpfile_new

```c
AU_PUBLIC int au_tmpfile_new(struct au_tmpfile *tmp);
```

Defined in *src/platform/tmpfile.h*.

Creates a new temporary file with the `.c` extension

#### Arguments

*none*

#### Return value

*none*

### au_value_struct

```c
static au_value_t au_value_struct(struct au_struct *data);
```

Defined in *src/core/rt/value/main.h*.

Creates a value from a pointer to a structure. The pointer must be allocated using au_obj_malloc.

#### Arguments

*none*

#### Return value

*none*

### au_vm_exec_unverified

```c
AU_PRIVATE au_value_t au_vm_exec_unverified(
    struct au_vm_thread_local *tl, const struct au_bc_storage *bcs,
    const struct au_program_data *p_data, const au_value_t *args);
```

Defined in *src/core/vm/vm.h*.

Executes unverified bytecode in a au_bc_storage

#### Arguments

 * **tl:** thread local storage
 * **bcs:** the au_bc_storage to be executed. Bytecode stored here is unverified and should be checked beforehand if it's safe to run.
 * **p_data:** global program data
 * **args:** argument array
 * **link:** link to previous frame or imported program

#### Return value

return value specified by interpreted aument's return statement

### au_vm_exec_unverified_main

```c
AU_PUBLIC au_value_t au_vm_exec_unverified_main(
    struct au_vm_thread_local *tl, struct au_program *program);
```

Defined in *src/core/vm/vm.h*.

Executes unverified bytecode in a au_program

#### Arguments

 * **tl:** thread local storage
 * **program:** the au_program to be executed. Bytecode stored here is unverified and should be checked beforehand if it's safe to run.

#### Return value

return value

### au_vm_thread_local_del

```c
AU_PUBLIC void au_vm_thread_local_del(struct au_vm_thread_local *tl);
```

Defined in *src/core/vm/tl.h*.

Deinitializes an au_vm_thread_local instance

#### Arguments

 * **tl:** instance to be deinitialized

#### Return value

*none*

### au_vm_thread_local_get

```c
AU_PUBLIC struct au_vm_thread_local *au_vm_thread_local_get();
```

Defined in *src/core/vm/tl.h*.

Gets the current thread's au_vm_thread_local instance

#### Arguments

*none*

#### Return value

the current thread's au_vm_thread_local instance

### au_vm_thread_local_init

```c
AU_PUBLIC void
au_vm_thread_local_init(struct au_vm_thread_local *tl,
                        const struct au_program_data *p_data);
```

Defined in *src/core/vm/tl.h*.

Initializes an au_vm_thread_local instance

#### Arguments

 * **tl:** instance to be initialized
 * **p_data:** global au_program_data instance

#### Return value

*none*

### au_vm_thread_local_set

```c
AU_PUBLIC void au_vm_thread_local_set(struct au_vm_thread_local *tl);
```

Defined in *src/core/vm/tl.h*.

Sets the current thread's au_vm_thread_local instance. The caller must ensure that tl lasts for the lifetime of an executed aument program.

#### Arguments

 * **tl:** the current thread's au_vm_thread_local instance

#### Return value

*none*

## Structures

### au_dispatch_func

A dispatch method

```c
struct au_dispatch_func {
    int32_t num_args;
    struct au_dispatch_func_instance_array data;
    size_t fallback_fn;
};
```

Defined in *src/core/fn/main.h*.

### au_error_location

A structure representing where an error occurs.

```c
struct au_error_location {
    const char *path;
    const char *src;
    size_t len;
};
```

Defined in *src/core/int_error/error_location.h*.

### au_fn

Representation of an Aument function, used by the parser and the virtual machine.

```c
struct au_fn {
    enum au_fn_type type;
    uint32_t flags;
    union {
        struct au_lib_func lib_func;
        struct au_bc_storage bc_func;
        struct au_imported_func imported_func;
        struct au_none_func none_func;
        struct au_dispatch_func dispatch_func;
    } as;
};
```

Defined in *src/core/fn/main.h*.

### au_imported_func

An imported function. This object stores information about a possibly unresolved function imported from another module.

```c
struct au_imported_func {
    int32_t num_args;
    int32_t module_idx;
    char *name;
    size_t name_len;
    size_t fn_idx_cached;
    const struct au_program_data *p_data_cached;
};
```

Defined in *src/core/fn/main.h*.

### au_none_func

A forward declared function. This object must only appear while parsing, and must not appear anywhere else.

```c
struct au_none_func {
    /// Expected number of arguments this function has
    int32_t num_args;
    /// Expected name of the function
    struct au_token name_token;
};
```

Defined in *src/core/fn/main.h*.
